import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

public class UE11Aufgabe24 {

	public static void main(String[] args) {
		String filename = "Board2.rh";

		RHGame game = new RHGame(filename);
		game.play();
	}
}

class RHGame {

	RHBoard board;

	RHGame(String filename) {
		this.board = new RHBoard(filename);
	}

	void play() {
		this.board.print();
		RHCar car = this.chooseCar();
		while (true) {
			char direction = IO
					.readChar("Richtung (l=links, r=rechts, u=hoch, d=runter, q=Autowechsel): ");
			chooseDirection: while (direction != 'q') {
				switch (direction) {
				case 'l':
					car.moveLeft();
					break;
				case 'r':
					car.moveRight();
					break;
				case 'u':
					car.moveUpwards();
					break;
				case 'd':
					car.moveDownwards();
					break;
				case 'q':
					break chooseDirection;
				default:
					System.out.println("Fehlerhafte Richtungswahl!");
					break;
				}
				this.board.print();
				if (this.board.checkEnd()) {
					System.out.println("Glckwunsch!");
					return;
				}
				direction = IO
						.readChar("Richtung (l=links, r=rechts, u=hoch, d=runter, q=Autowechsel): ");
			}
			car = this.chooseCar();
		}
	}

	RHCar chooseCar() {
		char carSign = IO.readChar("Wahl eines Autos (Buchstabe angeben): ");
		RHCar car = this.board.getCar(carSign);
		while (car == null) {
			carSign = IO
					.readChar("Fehler! Wahl eines Autos (Buchstabe angeben): ");
			car = this.board.getCar(carSign);
		}
		return car;
	}

}

class RHBoard {

	final static char EMPTY = ' ';

	int rows;
	int cols;
	RHCar[] cars;
	RHCar mainCar;

	RHBoard(String filename) {
		this.rows = 0;
		this.cols = 0;
		this.cars = null;
		this.mainCar = null;
		this.init(filename);
	}

	int getRows() {
		return this.rows;
	}

	int getCols() {
		return this.cols;
	}

	RHCar[] getCars() {
		return this.cars;
	}

	RHCar getMainCar() {
		return this.mainCar;
	}

	void print() {
		for (int r = 0; r < this.rows; r++) {
			for (int c = 0; c < this.cols; c++) {
				System.out.print(this.getBoardSign(r, c));
			}
			System.out.println();
		}
		System.out.println();
	}

	char getBoardSign(int row, int col) {
		for (int c = 0; c < this.cars.length; c++) {
			if (this.cars[c].covers(row, col)) {
				return this.cars[c].getSign();
			}
		}
		return RHBoard.EMPTY;
	}

	boolean checkEnd() {
		return this.mainCar.touchesRightBorder();
	}

	RHCar getCar(char sign) {
		for (RHCar car : this.cars) {
			if (car.getSign() == sign) {
				return car;
			}
		}
		return null;
	}

	boolean init(String filename) {
		char[][] chars = IO.readFileAsCharMatrix(filename);

		if (chars == null) {
			return false;
		}
		if (!RHBoard.checkFileContent(chars)) {
			return false;
		}
		this.rows = chars.length;
		this.cols = chars[0].length;
		for (int i = 1; i < chars.length; i++) {
			if (chars[i].length > this.cols) {
				this.cols = chars[i].length;
			}
		}
		this.createCars(chars);
		return true;
	}

	void createCars(char[][] chars) {
		// berechne Anzahl an Autos
		HashSet<Character> carSigns = new HashSet<Character>();
		for (int r = 0; r < chars.length; r++) {
			for (int c = 0; c < chars[r].length; c++) {
				if (chars[r][c] != RHBoard.EMPTY) {
					carSigns.add(chars[r][c]);
				}
			}
		}
		int noofCars = carSigns.size();

		// erstelle die Autos
		HashMap<Character, ArrayList<RHCarPart>> carMap = new HashMap<Character, ArrayList<RHCarPart>>();
		for (char ch : carSigns) {
			carMap.put(ch, new ArrayList<RHCarPart>());
		}
		for (int r = 0; r < chars.length; r++) {
			for (int c = 0; c < chars[r].length; c++) {
				if (chars[r][c] != RHBoard.EMPTY) {
					carMap.get(chars[r][c]).add(
							new RHCarPart(r, c, chars[r][c]));
				}
			}
		}
		this.cars = new RHCar[noofCars];
		int n = 0;
		for (ArrayList<RHCarPart> carParts : carMap.values()) {
			this.cars[n] = new RHCar(carParts.get(0).getSign(),
					carParts.toArray(new RHCarPart[0]), this);
			if (this.cars[n].getSign() == RHCar.MAIN) {
				this.mainCar = this.cars[n];
			}
			n++;
		}
	}

	static boolean checkFileContent(char[][] chars) {
		// wir gehen davon aus, dass die Datei korrekt ist
		return true;
	}

}

class RHCar {

	final static char MAIN = '*';

	char sign;
	RHCarPart[] parts;
	RHBoard board;

	RHCar(char sign, RHCarPart[] parts, RHBoard board) {
		this.sign = sign;
		this.parts = parts;
		this.board = board;
	}

	char getSign() {
		return this.sign;
	}

	boolean moveLeft() {
		if (!this.horizontalAlignment() || this.touchesLeftBorder()
				|| this.touchesCarLeft()) {
			return false;
		}
		for (RHCarPart part : this.parts) {
			part.moveLeft();
		}
		return true;
	}

	boolean moveRight() {
		if (!this.horizontalAlignment() || this.touchesRightBorder()
				|| this.touchesCarRight()) {
			return false;
		}
		for (RHCarPart part : this.parts) {
			part.moveRight();
		}
		return true;
	}

	boolean moveUpwards() {
		if (this.horizontalAlignment() || this.touchesTopBorder()
				|| this.touchesCarUp()) {
			return false;
		}
		for (RHCarPart part : this.parts) {
			part.moveUpwards();
		}
		return true;
	}

	boolean moveDownwards() {
		if (this.horizontalAlignment() || this.touchesBottomBorder()
				|| this.touchesCarDown()) {
			return false;
		}
		for (RHCarPart part : this.parts) {
			part.moveDownwards();
		}
		return true;
	}

	boolean touchesCarLeft() {
		RHCar[] cars = this.board.getCars();
		for (RHCarPart part : this.parts) {
			for (RHCar car : cars) {
				if (car != this && car.covers(part.getRow(), part.getCol() - 1)) {
					return true;
				}
			}
		}
		return false;
	}

	boolean touchesCarRight() {
		RHCar[] cars = this.board.getCars();
		for (RHCarPart part : this.parts) {
			for (RHCar car : cars) {
				if (car != this && car.covers(part.getRow(), part.getCol() + 1)) {
					return true;
				}
			}
		}
		return false;
	}

	boolean touchesCarUp() {
		RHCar[] cars = this.board.getCars();
		for (RHCarPart part : this.parts) {
			for (RHCar car : cars) {
				if (car != this && car.covers(part.getRow() - 1, part.getCol())) {
					return true;
				}
			}
		}
		return false;
	}

	boolean touchesCarDown() {
		RHCar[] cars = this.board.getCars();
		for (RHCarPart part : this.parts) {
			for (RHCar car : cars) {
				if (car != this && car.covers(part.getRow() + 1, part.getCol())) {
					return true;
				}
			}
		}
		return false;
	}

	boolean touchesLeftBorder() {
		for (RHCarPart part : this.parts) {
			if (part.getCol() == 0) {
				return true;
			}
		}
		return false;
	}

	boolean touchesRightBorder() {
		for (RHCarPart part : this.parts) {
			if (part.getCol() == this.board.getCols() - 1) {
				return true;
			}
		}
		return false;
	}

	boolean touchesTopBorder() {
		for (RHCarPart part : this.parts) {
			if (part.getRow() == 0) {
				return true;
			}
		}
		return false;
	}

	boolean touchesBottomBorder() {
		for (RHCarPart part : this.parts) {
			if (part.getRow() == this.board.getRows() - 1) {
				return true;
			}
		}
		return false;
	}

	boolean covers(int row, int col) {
		for (RHCarPart p : this.parts) {
			if (p.getRow() == row && p.getCol() == col) {
				return true;
			}
		}
		return false;
	}

	boolean horizontalAlignment() {
		return this.parts[0].getRow() == this.parts[1].getRow();
	}
}

class RHCarPart {
	int row;
	int col;
	char sign;

	RHCarPart(int row, int col, char sign) {
		this.row = row;
		this.col = col;
		this.sign = sign;
	}

	int getRow() {
		return this.row;
	}

	int getCol() {
		return this.col;
	}

	char getSign() {
		return this.sign;
	}

	void moveLeft() {
		this.col--;
	}

	void moveRight() {
		this.col++;
	}

	void moveUpwards() {
		this.row--;
	}

	void moveDownwards() {
		this.row++;
	}
}
